Use "#if" (not "#ifdef") for configure-defined macros.
[rsync.git] / main.c
diff --git a/main.c b/main.c
index 03af24bb646e524d06e90b17701f740d9534ad79..8ba45a262d15da75ed0b0edb457e506e1f084ae5 100644 (file)
--- a/main.c
+++ b/main.c
@@ -33,8 +33,9 @@ extern int verbose;
 extern int blocking_io;
 extern int cvs_exclude;
 extern int delete_mode;
-extern int delete_excluded;
+extern int delete_before;
 extern int delete_after;
+extern int delete_excluded;
 extern int daemon_over_rsh;
 extern int do_stats;
 extern int dry_run;
@@ -59,6 +60,8 @@ extern int filesfrom_fd;
 extern pid_t cleanup_child_pid;
 extern char *files_from;
 extern char *remote_filesfrom_file;
+extern char *partial_dir;
+extern char *basis_dir[];
 extern char *rsync_path;
 extern char *shell_cmd;
 extern char *batch_name;
@@ -145,6 +148,10 @@ static void report(int f)
                        write_longint(f, total_read);
                        write_longint(f, total_written);
                        write_longint(f, stats.total_size);
+                       if (protocol_version >= 29) {
+                               write_longint(f, stats.flist_buildtime);
+                               write_longint(f, stats.flist_xfertime);
+                       }
                }
                return;
        }
@@ -157,12 +164,20 @@ static void report(int f)
                total_written = read_longint(f);
                total_read = read_longint(f);
                stats.total_size = read_longint(f);
+               if (protocol_version >= 29) {
+                       stats.flist_buildtime = read_longint(f);
+                       stats.flist_xfertime = read_longint(f);
+               }
        } else if (write_batch) {
                /* The --read-batch process is going to be a client
                 * receiver, so we need to give it the stats. */
                write_longint(batch_fd, total_read);
                write_longint(batch_fd, total_written);
                write_longint(batch_fd, stats.total_size);
+               if (protocol_version >= 29) {
+                       write_longint(batch_fd, stats.flist_buildtime);
+                       write_longint(batch_fd, stats.flist_xfertime);
+               }
        }
 
        if (do_stats) {
@@ -178,6 +193,14 @@ static void report(int f)
                rprintf(FINFO,"Matched data: %.0f bytes\n",
                        (double)stats.matched_data);
                rprintf(FINFO,"File list size: %d\n", stats.flist_size);
+               if (stats.flist_buildtime) {
+                       rprintf(FINFO,
+                               "File list generation time: %.3f seconds\n",
+                               (double)stats.flist_buildtime / 1000);
+                       rprintf(FINFO,
+                               "File list transfer time: %.3f seconds\n",
+                               (double)stats.flist_xfertime / 1000);
+               }
                rprintf(FINFO,"Total bytes sent: %.0f\n",
                        (double)total_written);
                rprintf(FINFO,"Total bytes received: %.0f\n",
@@ -204,7 +227,7 @@ static void report(int f)
  **/
 static void show_malloc_stats(void)
 {
-#ifdef HAVE_MALLINFO
+#if HAVE_MALLINFO
        struct mallinfo mi;
 
        mi = mallinfo();
@@ -431,7 +454,7 @@ static void do_server_sender(int f_in, int f_out, int argc,char *argv[])
                        argv[i] += l+1;
        }
 
-       if (argc == 0 && recurse) {
+       if (argc == 0 && (recurse || list_only)) {
                argc = 1;
                argv--;
                argv[0] = ".";
@@ -461,7 +484,8 @@ static int do_recv(int f_in,int f_out,struct file_list *flist,char *local_name)
 {
        int pid;
        int status = 0;
-       int error_pipe[2];
+       int error_pipe[2], name_pipe[2];
+       BOOL need_name_pipe = (basis_dir[0] || partial_dir) && !dry_run;
 
        /* The receiving side mustn't obey this, or an existing symlink that
         * points to an identical file won't be replaced by the referent. */
@@ -470,21 +494,32 @@ static int do_recv(int f_in,int f_out,struct file_list *flist,char *local_name)
        if (preserve_hard_links)
                init_hard_links(flist);
 
-       if (!delete_after) {
+       if (delete_before) {
                /* I moved this here from recv_files() to prevent a race condition */
-               if (recurse && delete_mode && !local_name && flist->count > 0)
+               if (recurse && !local_name && flist->count > 0)
                        delete_files(flist);
        }
 
-       if (fd_pair(error_pipe) < 0) {
-               rprintf(FERROR,"error pipe failed in do_recv\n");
-               exit_cleanup(RERR_SOCKETIO);
+       if (fd_pair(error_pipe) < 0
+           || (need_name_pipe && fd_pair(name_pipe) < 0)) {
+               rsyserr(FERROR, errno, "pipe failed in do_recv");
+               exit_cleanup(RERR_IPC);
        }
 
        io_flush(NORMAL_FLUSH);
 
-       if ((pid = do_fork()) == 0) {
+       if ((pid = do_fork()) == -1) {
+               rsyserr(FERROR, errno, "fork failed in do_recv");
+               exit_cleanup(RERR_IPC);
+       }
+
+       if (pid == 0) {
                close(error_pipe[0]);
+               if (need_name_pipe) {
+                       close(name_pipe[1]);
+                       set_blocking(name_pipe[0]);
+               } else
+                       name_pipe[0] = -1;
                if (f_in != f_out)
                        close(f_out);
 
@@ -494,7 +529,7 @@ static int do_recv(int f_in,int f_out,struct file_list *flist,char *local_name)
                /* set place to send errors */
                set_msg_fd_out(error_pipe[1]);
 
-               recv_files(f_in,flist,local_name);
+               recv_files(f_in, flist, local_name, name_pipe[0]);
                io_flush(FULL_FLUSH);
                report(f_in);
 
@@ -513,6 +548,11 @@ static int do_recv(int f_in,int f_out,struct file_list *flist,char *local_name)
                stop_write_batch();
 
        close(error_pipe[1]);
+       if (need_name_pipe) {
+               close(name_pipe[0]);
+               set_nonblocking(name_pipe[1]);
+       } else
+               name_pipe[1] = -1;
        if (f_in != f_out)
                close(f_in);
 
@@ -520,9 +560,8 @@ static int do_recv(int f_in,int f_out,struct file_list *flist,char *local_name)
 
        set_msg_fd_in(error_pipe[0]);
 
-       generate_files(f_out, flist, local_name);
+       generate_files(f_out, flist, local_name, name_pipe[1]);
 
-       get_redo_num(); /* Read final MSG_DONE and any prior messages. */
        report(-1);
        io_flush(FULL_FLUSH);
        if (protocol_version >= 24) {
@@ -544,6 +583,13 @@ static void do_server_recv(int f_in, int f_out, int argc,char *argv[])
        struct file_list *flist;
        char *local_name = NULL;
        char *dir = NULL;
+       int save_verbose = verbose;
+
+       if (filesfrom_fd >= 0) {
+               /* We can't mix messages with files-from data on the socket,
+                * so temporarily turn off verbose messages. */
+               verbose = 0;
+       }
 
        if (verbose > 2) {
                rprintf(FINFO, "server_recv(%d) starting pid=%ld\n",
@@ -570,18 +616,22 @@ static void do_server_recv(int f_in, int f_out, int argc,char *argv[])
 
        io_start_buffering_in();
        if (delete_mode && !delete_excluded)
-               recv_exclude_list(f_in);
+               recv_filter_list(f_in);
+       if (cvs_exclude)
+               add_cvs_excludes();
 
        if (filesfrom_fd >= 0) {
/* We're receiving the file info from the sender, so we need
* the IO routines to automatically write out the names onto
* our f_out socket as we read the list info from the sender.
* This avoids both deadlock and extra delays/buffers. */
                io_set_filesfrom_fds(filesfrom_fd, f_out);
                filesfrom_fd = -1;
        }
 
        flist = recv_file_list(f_in);
+       verbose = save_verbose;
        if (!flist) {
                rprintf(FERROR,"server_recv: recv_file_list error\n");
                exit_cleanup(RERR_FILESELECT);
@@ -622,7 +672,7 @@ void start_server(int f_in, int f_out, int argc, char *argv[])
        if (am_sender) {
                keep_dirlinks = 0; /* Must be disabled on the sender. */
 
-               recv_exclude_list(f_in);
+               recv_filter_list(f_in);
                if (cvs_exclude)
                        add_cvs_excludes();
                do_server_sender(f_in, f_out, argc, argv);
@@ -644,9 +694,7 @@ int client_run(int f_in, int f_out, pid_t pid, int argc, char *argv[])
        char *local_name = NULL;
 
        cleanup_child_pid = pid;
-       if (read_batch) {
-               assert(am_sender == 0);
-       } else {
+       if (!read_batch) {
                set_nonblocking(f_in);
                set_nonblocking(f_out);
        }
@@ -670,10 +718,12 @@ int client_run(int f_in, int f_out, pid_t pid, int argc, char *argv[])
        if (am_sender) {
                keep_dirlinks = 0; /* Must be disabled on the sender. */
                io_start_buffering_out();
+               if (!remote_filesfrom_file)
+                       set_msg_fd_in(f_in);
                if (cvs_exclude)
                        add_cvs_excludes();
                if (delete_mode && !delete_excluded)
-                       send_exclude_list(f_out);
+                       send_filter_list(f_out);
                if (remote_filesfrom_file)
                        filesfrom_fd = f_in;
 
@@ -681,6 +731,7 @@ int client_run(int f_in, int f_out, pid_t pid, int argc, char *argv[])
                        start_write_batch(f_out);
                if (!read_batch) /* don't write to pipe */
                        flist = send_file_list(f_out,argc,argv);
+               set_msg_fd_in(-1);
                if (verbose > 3)
                        rprintf(FINFO,"file list sent\n");
 
@@ -703,10 +754,12 @@ int client_run(int f_in, int f_out, pid_t pid, int argc, char *argv[])
        }
 
        if (argc == 0)
-               list_only = 1;
+               list_only |= 1;
 
        if (!read_batch)
-               send_exclude_list(f_out);
+               send_filter_list(f_out);
+       if (cvs_exclude)
+               add_cvs_excludes();
 
        if (filesfrom_fd >= 0) {
                io_set_filesfrom_fds(filesfrom_fd, f_out);
@@ -934,9 +987,8 @@ static int start_client(int argc, char *argv[])
        }
 
        /* ... or no dest at all */
-       if (!am_sender && argc == 0) {
-               list_only = 1;
-       }
+       if (!am_sender && argc == 0)
+               list_only |= 1;
 
        pid = do_cmd(shell_cmd,shell_machine,shell_user,shell_path,
                     &f_in,&f_out);
@@ -570,18 +616,22 @@ static void do_server_recv(int f_in, int f_out, int argc,char *argv[])
 
        if (filesfrom_fd >= 0) {
-               /* We're receiving the file info from the sender, so we need
-                * the IO routines to automatically write out the names onto
-                * our f_out socket as we read the list info from the sender.
-                * This avoids both deadlock and extra delays/buffers. */
+               /* We need to send the files-from names to the sender at the
+                * same time that we receive the file-list from them, so we
+                * need the IO routines to automatically write out the names
+                * onto our f_out socket as we read the file-list.  This
+                * avoids both deadlock and extra delays/buffers. */
                io_set_filesfrom_fds(filesfrom_fd, f_out);
                filesfrom_fd = -1;